@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/imagefile/imagefile.h>
+#include <mobius/vfs/imagefile.h>
 #include "imagefile_impl.h"
 #include <cstdint>
 
@@ -261,7 +261,7 @@ std::uint32_t EXTENSION_ICON_DATA_SIZE = sizeof (EXTENSION_ICON_DATA);
 extern "C" void
 start ()
 {
-  mobius::imagefile::add_implementation <mobius::raw::imagefile_impl> ();
+  mobius::vfs::add_imagefile_implementation <imagefile_impl> ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -270,5 +270,5 @@ start ()
 extern "C" void
 stop ()
 {
-  mobius::imagefile::remove_implementation <mobius::raw::imagefile_impl> ();
+  mobius::vfs::remove_imagefile_implementation <imagefile_impl> ();
 }
