@@ -45,10 +45,10 @@ static const std::string id_ = "mobiusft";
 static const std::string name_ = "Mobius Forensic Toolkit";
 
 //! \brief application version
-static const std::string version_ = "1.27";
+static const std::string version_ = "1.33";
 
 //! \brief application title
-static const std::string title_ = "Mobius v1.27";
+static const std::string title_ = "Mobius v1.33";
 
 //! \brief application copyright
 static const std::string copyright_ = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar";
@@ -232,7 +232,7 @@ application::stop ()
     e.stop ();
 
   // remove extensions
-  extensions_.clear ();
+  //extensions_.clear ();
 
   // set running false  
   log.info (__LINE__, "application stopped");
