@@ -17,11 +17,12 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include "item.h"
 #include "case.h"
-#include <mobius/exception.inc>
 #include <mobius/core/category_manager.h>
+#include <mobius/exception.inc>
 #include <mobius/io/folder.h>
-#include <stdexcept>
 #include <cstddef>
+#include <stdexcept>
+#include <unordered_map>
 
 namespace mobius
 {
@@ -68,7 +69,11 @@ expand_value_mask (const std::string& value_mask, mobius::model::item item)
           else
             {
               std::string var = value_mask.substr (start_pos + 2, end_pos - start_pos - 2);
-              out += item.get_attribute (var);
+              auto value = item.get_attribute (var);
+
+              if (!value.is_null ())
+                out += item.get_attribute (var).to_string ();
+
               pos = end_pos + 1;
             }
         }
@@ -102,6 +107,11 @@ public:
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // function prototypes
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  bool has_attribute (const std::string&) const;
+  mobius::pod::data get_attribute (const std::string&) const;
+  void set_attribute (const std::string&, const mobius::pod::data&);
+  void remove_attribute (const std::string&);
+
   item new_child (const std::string&, int);
   void remove ();
   void move (int, const item&);
@@ -128,40 +138,6 @@ public:
     return category_;
   }
 
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  //! \brief Get metadata
-  //! \return Metadata
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  mobius::pod::data
-  get_metadata () const
-  {
-    _load_data ();
-    return metadata_;
-  }
-
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  //! \brief Set metadata
-  //! \param metadata Metadata
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  void
-  set_metadata (const mobius::pod::data& metadata)
-  {
-    _load_data ();
-   
-    auto db = get_database ();
-
-    auto stmt = db.new_statement (
-               "UPDATE item "
-                  "SET metadata = ? "
-                "WHERE uid = ?");
-
-    stmt.bind (1, metadata);
-    stmt.bind (2, uid_);
-    stmt.execute ();
-    
-    metadata_ = metadata;
-  }
-
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   //! \brief Get case
   //! \return case
@@ -182,6 +158,17 @@ public:
     return case_.get_database ();
   }
 
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  //! \brief Get attributes
+  //! \return Attributes
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  std::unordered_map <std::string, mobius::pod::data>
+  get_attributes () const
+  {
+    _load_attributes ();
+    return attributes_;
+  }
+
 private:
   //! \brief Case object
   Case case_;
@@ -192,16 +179,20 @@ private:
   //! \brief Category
   mutable std::string category_;
 
-  //! \brief Metadata
-  mutable mobius::pod::data metadata_;
+  //! \brief Attributes
+  mutable std::unordered_map <std::string, mobius::pod::data> attributes_;
 
   //! \brief Data loaded flag
   mutable bool data_loaded_ = false;
 
+  //! \brief Attributes loaded flag
+  mutable bool attributes_loaded_ = false;
+
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Helper functions
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   void _load_data () const;
+  void _load_attributes () const;
   int _reserve_index (int) const;
 };
 
@@ -241,6 +232,104 @@ item::impl::get_data_path (const std::string& rpath) const
   return case_.get_path ("data/" + std::string (s_uid) + '/' + rpath);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if item has attribute
+//! \param id Attribute ID
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+item::impl::has_attribute (const std::string& id) const
+{
+  _load_attributes ();
+
+  return attributes_.find (id) != attributes_.end ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get attribute
+//! \param id Attribute ID
+//! \return Attribute value or pod::null if not found
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::pod::data
+item::impl::get_attribute (const std::string& id) const
+{
+  mobius::pod::data value;
+
+  _load_attributes ();
+
+  auto iter = attributes_.find (id);
+  
+  if (iter != attributes_.end ())
+    value = iter->second;
+
+  return value;	
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set attribute
+//! \param id Attribute ID
+//! \param value Attribute value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::impl::set_attribute (const std::string& id, const mobius::pod::data& value)
+{
+  // Add to database
+  auto db = get_database ();
+  mobius::database::statement stmt;
+
+  if (has_attribute (id))
+    {
+      stmt = db.new_statement (
+               "UPDATE attribute "
+                  "SET value = ? "
+                "WHERE item_uid = ? "
+                  "AND id = ?");
+
+      stmt.bind (1, value);
+      stmt.bind (2, uid_);
+      stmt.bind (3, id);
+    }
+
+  else
+    {
+      stmt = db.new_statement (
+               "INSERT INTO attribute "
+                    "VALUES (NULL, ?, ?, ?)");
+
+      stmt.bind (1, uid_);
+      stmt.bind (2, id);
+      stmt.bind (3, value);
+    }
+
+  stmt.execute ();
+  
+  // Add to cache
+  attributes_[id] = value;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Remove attribute
+//! \param id Attribute ID
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::impl::remove_attribute (const std::string& id)
+{
+  // Remove from database
+  auto db = get_database ();
+
+  auto stmt = db.new_statement (
+                "DELETE FROM attribute "
+                      "WHERE item_uid = ? "
+                        "AND id = ?");
+
+  stmt.bind (1, get_uid ());
+  stmt.bind (2, id);
+  stmt.execute ();
+
+  // Remove from cache
+  attributes_.erase (id);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create new child item
 //! \param category item's category
@@ -265,7 +354,7 @@ item::impl::new_child (const std::string& category, int idx)
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   auto stmt = db.new_statement (
                 "INSERT INTO item "
-                     "VALUES (NULL, ?, ?, ?, DATETIME ('NOW'), NULL)");
+                     "VALUES (NULL, ?, ?, ?, DATETIME ('NOW'))");
 
   stmt.bind (1, uid_);
   stmt.bind (2, idx);
@@ -428,21 +517,47 @@ item::impl::_load_data () const
   auto db = case_.get_database ();
 
   auto stmt = db.new_statement (
-                "SELECT category, metadata "
+                "SELECT category "
                   "FROM item "
                  "WHERE uid = ?");
 
   stmt.bind (1, uid_);
 
   if (stmt.fetch_row ())
-    {
-      category_ = stmt.get_column_string (0);
-      metadata_ = stmt.get_column_pod (1);
-    }
+    category_ = stmt.get_column_string (0);
 
   data_loaded_ = true;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Load attributes on demand
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::impl::_load_attributes () const
+{
+  if (attributes_loaded_)
+    return;
+
+  // load attributes
+  auto db = get_database ();
+
+  auto stmt = db.new_statement (
+                "SELECT id, value "
+                  "FROM attribute "
+                 "WHERE item_uid = ?");
+
+  stmt.bind (1, uid_);
+
+  while (stmt.fetch_row ())
+    {
+      auto id = stmt.get_column_string (0);
+      auto value = stmt.get_column_pod (1);
+      attributes_[id] = value;
+    }
+
+  attributes_loaded_ = true;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Reserve one slot for a child at position idx
 //! \param idx index, starting in 1 or -1 for last position
@@ -558,6 +673,74 @@ item::get_case () const
   return impl_->get_case ();
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if attribute exists
+//! \param id attribute ID
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+item::has_attribute (const std::string& id) const
+{
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
+  return impl_->has_attribute (id);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get attribute value
+//! \param id attribute ID
+//! \return attribute value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::pod::data
+item::get_attribute (const std::string& id) const
+{
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
+  return impl_->get_attribute (id);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set attribute value
+//! \param id attribute ID
+//! \param value attribute value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::set_attribute (const std::string& id, const mobius::pod::data& value)
+{
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
+  impl_->set_attribute (id, value);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Remove attribute
+//! \param id attribute ID
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::remove_attribute (const std::string& id)
+{
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
+  impl_->remove_attribute (id);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get attributes
+//! \return Map containing attributes' IDs and values
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::unordered_map <std::string, mobius::pod::data>
+item::get_attributes () const
+{
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
+  return impl_->get_attributes ();
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get number of children
 //! \return Number of children
@@ -703,35 +886,12 @@ item::remove ()
 void
 item::move (int idx, const item& parent)
 {
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
+
   impl_->move (idx, parent);
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Get metadata
-//! \return Metadata
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::pod::data
-item::get_metadata () const
-{
-  if (!impl_)
-    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
-
-  return impl_->get_metadata ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Set metadata
-//! \param metadata Metadata
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-item::set_metadata (const mobius::pod::data& metadata)
-{
-  if (!impl_)
-    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
-
-  impl_->set_metadata (metadata);
-}
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Expand ${} masks into attribute values
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
