@@ -83,7 +83,7 @@ tp_getter_header_size (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -105,7 +105,7 @@ tp_getter_clsid (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -127,7 +127,7 @@ tp_getter_flags (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -149,7 +149,7 @@ tp_getter_file_attributes (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -171,7 +171,7 @@ tp_getter_creation_time (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -193,7 +193,7 @@ tp_getter_access_time (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -215,7 +215,7 @@ tp_getter_write_time (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -237,7 +237,7 @@ tp_getter_file_size (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -259,7 +259,7 @@ tp_getter_icon_index (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -281,7 +281,7 @@ tp_getter_show_command (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -303,7 +303,7 @@ tp_getter_hotkeys (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -325,7 +325,7 @@ tp_getter_volume_label (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -347,7 +347,7 @@ tp_getter_drive_type (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -369,7 +369,7 @@ tp_getter_drive_serial_number (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -391,7 +391,7 @@ tp_getter_local_base_path (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -413,7 +413,7 @@ tp_getter_common_path_suffix (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -435,7 +435,7 @@ tp_getter_net_name (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -457,7 +457,7 @@ tp_getter_device_name (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -479,7 +479,7 @@ tp_getter_network_provider_type (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -501,7 +501,7 @@ tp_getter_name (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -523,7 +523,7 @@ tp_getter_relative_path (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -545,7 +545,7 @@ tp_getter_working_dir (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -567,7 +567,7 @@ tp_getter_arguments (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -589,7 +589,7 @@ tp_getter_icon_location (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -611,7 +611,7 @@ tp_getter_netbios_name (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -633,7 +633,7 @@ tp_getter_header_offset (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -655,7 +655,7 @@ tp_getter_link_target_id_list_offset (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -677,7 +677,7 @@ tp_getter_link_info_offset (decoder_lnk_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -905,7 +905,7 @@ tp_f_is_target_readonly (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -930,7 +930,7 @@ tp_f_is_target_hidden (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -955,7 +955,7 @@ tp_f_is_target_system (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -980,7 +980,7 @@ tp_f_is_target_directory (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1005,7 +1005,7 @@ tp_f_is_target_archive (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1030,7 +1030,7 @@ tp_f_is_target_temporary (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1055,7 +1055,7 @@ tp_f_is_target_sparse (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1080,7 +1080,7 @@ tp_f_is_target_reparse_point (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1105,7 +1105,7 @@ tp_f_is_target_compressed (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1130,7 +1130,7 @@ tp_f_is_target_offline (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1155,7 +1155,7 @@ tp_f_is_target_content_indexed (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1180,7 +1180,7 @@ tp_f_is_target_encrypted (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1205,7 +1205,7 @@ tp_f_is_run_in_separate_process (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1230,7 +1230,7 @@ tp_f_is_run_as_user (decoder_lnk_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // Return value
@@ -1348,7 +1348,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -1365,7 +1365,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
       catch (const std::exception& e)
         {
           Py_DECREF (ret);
-          PyErr_SetString (PyExc_Exception, e.what ());
+          mobius::py::set_runtime_error (e.what ());
           ret = nullptr;
         }
     }
@@ -1381,7 +1381,7 @@ static void
 tp_dealloc (decoder_lnk_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -1436,4 +1436,5 @@ PyTypeObject decoder_lnk_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
