@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -20,9 +20,12 @@
 //! \brief  C++ API module wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include "pymobius.h"
+#include <Python.h>
 #include "module.h"
+#include "pymobius.h"
+#include "pymodule.h"
 #include <datetime.h>
+#include <mobius/core/log.h>
 #include "api_dataholder.h"
 
 #include "core/module.h"
@@ -32,7 +35,7 @@
 #include "decoder/module.h"
 #include "disk/module.h"
 #include "encoder/module.h"
-#include "imagefile/module.h"
+#include "kff/module.h"
 #include "io/module.h"
 #include "model/module.h"
 #include "os/module.h"
@@ -40,48 +43,25 @@
 #include "pod/module.h"
 #include "system/module.h"
 #include "turing/module.h"
+#include "ui/module.h"
 #include "vfs/module.h"
 
-#include <functional>
-#include <stdexcept>
-#include <utility>
-
 namespace
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Module methods
+//! \brief Module definition structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static PyMethodDef module_methods[] =
+static PyModuleDef module_def =
 {
-  {
-    NULL,
-    NULL,
-    0,
-    NULL
-  } // sentinel
-};
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Sub-modules
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static std::pair <std::string, std::function<PyObject *()>> sub_modules[] =
-{
-  {"core", new_core_module },
-  {"crypt", new_crypt_module },
-  {"database", new_database_module },
-  {"datetime", new_datetime_module },
-  {"decoder", new_decoder_module },
-  {"disk", new_disk_module },
-  {"encoder", new_encoder_module },
-  {"imagefile", new_imagefile_module },
-  {"io", new_io_module },
-  {"model", new_model_module },
-  {"os", new_os_module },
-  {"partition", new_partition_module },
-  {"pod", new_pod_module },
-  {"system", new_system_module },
-  {"turing", new_turing_module },
-  {"vfs", new_vfs_module },
+  PyModuleDef_HEAD_INIT,
+  "mobius",
+  "Mobius Forensic Toolkit API wrapper",
+  -1,
+  nullptr,
+  nullptr,
+  nullptr,
+  nullptr,
+  nullptr
 };
 
 } // namespace
@@ -90,35 +70,48 @@ static std::pair <std::string, std::function<PyObject *()>> sub_modules[] =
 //! \brief Module initialisation function
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyMODINIT_FUNC
-initmobius ()
+PyInit_mobius (void)
 {
-  // Build types
-  if (PyType_Ready (&api_dataholder_t) < 0)
-    return;
-
-  // Initialize module
-  PyObject* mobius_module = Py_InitModule3 (
-                              "mobius",
-                              module_methods,
-                              "Mobius Forensic Toolkit API wrapper"
-                            );
-
-  if (mobius_module == nullptr)
-    return;
-
-  // Add types
   PyDateTime_IMPORT;
+  PyObject *ret = nullptr;
 
-  // Build submodules
-  for (const auto& p : sub_modules)
+  try
     {
-      PyObject *s_module = p.second ();
-      PyModule_AddObject (mobius_module, p.first.c_str (), s_module);
+      // Initialize module
+      mobius::py::pymodule module (&module_def);
+
+      // Add types
+      module.add_type ("dataholder", &api_dataholder_t);
+
+      // Build submodules
+      module.add_submodule ("core", new_core_module ());
+      module.add_submodule ("crypt", new_crypt_module ());
+      module.add_submodule ("database", new_database_module ());
+      module.add_submodule ("datetime", new_datetime_module ());
+      module.add_submodule ("decoder", new_decoder_module ());
+      module.add_submodule ("disk", new_disk_module ());
+      module.add_submodule ("encoder", new_encoder_module ());
+      module.add_submodule ("kff", new_kff_module ());
+      module.add_submodule ("io", new_io_module ());
+      module.add_submodule ("model", new_model_module ());
+      module.add_submodule ("os", new_os_module ());
+      module.add_submodule ("partition", new_partition_module ());
+      module.add_submodule ("pod", new_pod_module ());
+      module.add_submodule ("system", new_system_module ());
+      module.add_submodule ("turing", new_turing_module ());
+      module.add_submodule ("ui", new_ui_module ());
+      module.add_submodule ("vfs", new_vfs_module ());
+
+      // Create new reference to module object
+      ret = module.new_reference ();
+    }
+  catch (const std::exception& e)
+    {
+      mobius::core::log log (__FILE__, __FUNCTION__);
+      log.error (__LINE__, e.what ());
     }
 
-  // deprecated: this types must be enclosed in submodules
-  Py_INCREF (&api_dataholder_t);
-  PyModule_AddObject (mobius_module, "dataholder", (PyObject *) &api_dataholder_t);
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -163,10 +156,7 @@ pydatetime_from_datetime (const mobius::datetime::datetime& dt)
     }
 
   else
-    {
-      Py_INCREF (Py_None);
-      ret = Py_None;
-    }
+    ret = mobius::py::pynone ();
 
   return ret;
 }
@@ -179,7 +169,7 @@ pydatetime_as_datetime (PyObject *obj)
 {
   mobius::datetime::datetime dt;
 
-  if (obj != Py_None)
+  if (!mobius::py::pynone_check (obj))
     dt = mobius::datetime::datetime (
            PyDateTime_GET_YEAR (obj),
            PyDateTime_GET_MONTH (obj),
@@ -193,12 +183,3 @@ pydatetime_as_datetime (PyObject *obj)
 
 } // namespace py
 } // namespace mobius
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-//! \brief Create PyString from std::string
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-PyObject *
-PyString2_from_std_string (const std::string& s)
-{
-  return PyString_FromString (s.c_str ());
-}
