@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -173,6 +173,34 @@ get_arg_as_bool (PyObject *args, std::uint32_t idx, bool value)
   return value;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as int
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+int
+get_arg_as_int (PyObject *args, std::uint32_t idx)
+{
+  return PyLong_AsLong (get_arg (args, idx));
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as int (with default value)
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \param value Default value
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+int
+get_arg_as_int (PyObject *args, std::uint32_t idx, int value)
+{
+  if (idx < get_arg_size (args))
+    return PyLong_AsLong (get_arg (args, idx));
+  
+  return value;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get function argument as std::int32_t
 //! \param args Argument tuple
@@ -229,6 +257,62 @@ get_arg_as_int64_t (PyObject *args, std::uint32_t idx, std::int64_t value)
   return value;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as std::uint8_t
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint8_t
+get_arg_as_uint8_t (PyObject *args, std::uint32_t idx)
+{
+  return pylong_as_std_uint8_t (get_arg (args, idx));
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as std::uint8_t (with default value)
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \param value Default value
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint8_t
+get_arg_as_uint8_t (PyObject *args, std::uint32_t idx, std::uint8_t value)
+{
+  if (idx < get_arg_size (args))
+    return get_arg_as_uint8_t (args, idx);
+  
+  return value;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as std::uint16_t
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint16_t
+get_arg_as_uint16_t (PyObject *args, std::uint32_t idx)
+{
+  return pylong_as_std_uint16_t (get_arg (args, idx));
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get function argument as std::uint16_t (with default value)
+//! \param args Argument tuple
+//! \param idx Argument index
+//! \param value Default value
+//! \return Argument
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint16_t
+get_arg_as_uint16_t (PyObject *args, std::uint32_t idx, std::uint16_t value)
+{
+  if (idx < get_arg_size (args))
+    return get_arg_as_uint16_t (args, idx);
+  
+  return value;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get function argument as std::uint32_t
 //! \param args Argument tuple
@@ -252,7 +336,7 @@ std::uint32_t
 get_arg_as_uint32_t (PyObject *args, std::uint32_t idx, std::uint32_t value)
 {
   if (idx < get_arg_size (args))
-    return pylong_as_std_uint32_t (get_arg (args, idx));
+    return get_arg_as_uint32_t (args, idx);
   
   return value;
 }
@@ -280,11 +364,79 @@ std::uint64_t
 get_arg_as_uint64_t (PyObject *args, std::uint32_t idx, std::uint64_t value)
 {
   if (idx < get_arg_size (args))
-    return pylong_as_std_uint64_t (get_arg (args, idx));
+    return get_arg_as_uint64_t (args, idx);
   
   return value;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Reset error
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+reset_error ()
+{
+  PyErr_Clear ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set AttributeError
+//! \param msg Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_attribute_error (const std::string& msg)
+{
+  PyErr_SetString (PyExc_AttributeError, msg.c_str ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set TypeError
+//! \param msg Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_invalid_type_error (const std::string& msg)
+{
+  PyErr_SetString (PyExc_TypeError, msg.c_str ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set ValueError
+//! \param msg Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_value_error (const std::string& msg)
+{
+  PyErr_SetString (PyExc_ValueError, msg.c_str ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set IOError
+//! \param msg Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_io_error (const std::string& msg)
+{
+  PyErr_SetString (PyExc_IOError, msg.c_str ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set RuntimeError
+//! \param msg Error message
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_runtime_error (const std::string& msg)
+{
+  PyErr_SetString (PyExc_RuntimeError, msg.c_str ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Set StopIteration
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+set_stop_iteration ()
+{
+  PyErr_SetNone (PyExc_StopIteration);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get current line number
 //! \return Line number
@@ -334,6 +486,17 @@ get_funcname ()
   return funcname;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if object is Py_None
+//! \param value Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+pynone_check (PyObject *value)
+{
+  return value == Py_None;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create new instance of Py_None
 //! \return Python object
@@ -353,11 +516,7 @@ pynone ()
 bool
 pystring_check (PyObject *value)
 {
-#if PY_MAJOR_VERSION < 3
-  return PyString_Check (value) || PyUnicode_Check (value);
-#else
   return PyUnicode_Check (value);
-#endif
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -379,24 +538,7 @@ pystring_from_std_string (const std::string& s)
 std::string
 pystring_as_std_string (PyObject *value)
 {
-#if PY_MAJOR_VERSION < 3
-  const char *buffer = nullptr;
-  PyObject *pystr = nullptr;
-
-  if (PyString_Check (value))
-    buffer = PyString_AsString (value);
-
-  else if (PyUnicode_Check (value))
-    {
-      pystr = PyUnicode_AsUTF8String (value);
-
-      if (pystr)
-        buffer = PyString_AsString (pystr);
-    }
-#else
   const char *buffer = PyUnicode_AsUTF8 (value);
-#endif
-
   std::string str;
 
   if (buffer)
@@ -405,11 +547,6 @@ pystring_as_std_string (PyObject *value)
   else
     throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("Invalid Python string"));
 
-#if PY_MAJOR_VERSION < 3
-  if (pystr)
-    Py_DECREF (pystr);
-#endif
-
   return str;
 }
 
@@ -447,7 +584,7 @@ pybytes_as_bytearray (PyObject *value)
   Py_ssize_t length;
 
   if (PyBytes_AsStringAndSize (value, &buffer, &length) == -1)
-    throw std::runtime_error (mobius::MOBIUS_EXCEPTION_MSG ("Invalid Python bytearray"));
+    throw std::runtime_error (mobius::MOBIUS_EXCEPTION_MSG ("invalid bytes"));
 
   return mobius::bytearray (reinterpret_cast <const std::uint8_t *> (buffer), length);
 }
@@ -500,6 +637,39 @@ pybool_as_bool (PyObject *value)
   return value == Py_True;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check PyLong
+//! \param value Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+pylong_check (PyObject *value)
+{
+  return PyLong_Check (value);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create PyLong from std::int8_t
+//! \param value Value
+//! \return Python Unicode object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+pylong_from_std_int8_t (std::int8_t value)
+{
+  return PyLong_FromLong (value);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create std::int8_t from PyLong
+//! \param value Python Object
+//! \return std::int8_t
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::int8_t
+pylong_as_std_int8_t (PyObject *value)
+{
+  return PyLong_AsLong (value);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create PyLong from std::int16_t
 //! \param value Value
@@ -544,10 +714,32 @@ pylong_as_std_int32_t (PyObject *value)
   return PyLong_AsLong (value);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create PyLong from std::uint8_t
+//! \param value Value
+//! \return Python Long object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+pylong_from_std_uint8_t (std::uint8_t value)
+{
+  return PyLong_FromUnsignedLong (value);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create PyLong from std::uint16_t
+//! \param value Value
+//! \return Python Long object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+pylong_from_std_uint16_t (std::uint16_t value)
+{
+  return PyLong_FromUnsignedLong (value);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create PyLong from std::uint32_t
 //! \param value Value
-//! \return Python Unicode object
+//! \return Python Long object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
 pylong_from_std_uint32_t (std::uint32_t value)
@@ -555,6 +747,28 @@ pylong_from_std_uint32_t (std::uint32_t value)
   return PyLong_FromUnsignedLong (value);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create std::uint8_t from PyLong
+//! \param value Python Object
+//! \return std::uint8_t
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint8_t
+pylong_as_std_uint8_t (PyObject *value)
+{
+  return PyLong_AsUnsignedLong (value);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create std::uint16_t from PyLong
+//! \param value Python Object
+//! \return std::uint16_t
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::uint16_t
+pylong_as_std_uint16_t (PyObject *value)
+{
+  return PyLong_AsUnsignedLong (value);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create std::uint32_t from PyLong
 //! \param value Python Object
@@ -563,11 +777,6 @@ pylong_from_std_uint32_t (std::uint32_t value)
 std::uint32_t
 pylong_as_std_uint32_t (PyObject *value)
 {
-#if PY_MAJOR_VERSION < 3
-  if (PyInt_Check (value))
-    return PyInt_AsUnsignedLongMask (value);
-#endif
-
   return PyLong_AsUnsignedLong (value);
 }
 
@@ -612,11 +821,6 @@ pylong_from_std_uint64_t (std::uint64_t value)
 std::uint64_t
 pylong_as_std_uint64_t (PyObject *value)
 {
-#if PY_MAJOR_VERSION < 3
-  if (PyInt_Check (value))
-    return PyInt_AsUnsignedLongLongMask (value);
-#endif
-
   return PyLong_AsUnsignedLongLong (value);
 }
 
