@@ -19,22 +19,18 @@ import os
 import os.path
 import tempfile
 import sqlite3
-import shutil
 import pymobius
 import mobius
 import re
-import decoder_calls
-import decoder_messages
+from . import decoder_calls
+from . import decoder_messages
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Decode %UUUU chars from string
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 def pct_decode (s):
   def repl (matchobj):
-    return unichr (int (matchobj.group (1), 16))
-
-  if isinstance (s, str):
-    s = s.decode ('utf-8')
+    return chr (int (matchobj.group (1), 16))
 
   s = re.sub ('%([0-9a-f]{4})', repl, s)
 
@@ -49,18 +45,18 @@ def get_schema_version (db):
 
   stmt = '''
        SELECT value
-         FROM metadata 
+         FROM metadata
         WHERE name  = 'schemaVersion'
         '''
 
   try:
     cursor = db.execute (stmt)
     row = cursor.fetchone ()
-    
+
     if row:
       version = int (row[0])
 
-  except Exception, e:
+  except Exception as e:
     mobius.core.logf ('WRN ' + str (e))
 
   return version
@@ -139,7 +135,7 @@ class Profile (object):
   # @brief Load account
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __load_account (self):
-    return 
+    return
     db = self.__get_db ()
 
     cursor = db.execute ('''
@@ -164,7 +160,7 @@ class Profile (object):
   # @todo Implement
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __load_calls (self):
-    if not self.__calls_loaded:          
+    if not self.__calls_loaded:
       db = self.__get_db ()
       self.__calls, skype_names = decoder_calls.decode (db, self.name)
       self.__skype_names.update (skype_names)
@@ -217,7 +213,7 @@ class Profile (object):
       ext = os.path.splitext (self.__f.name)[1]
       fd, path = tempfile.mkstemp (suffix=ext)
 
-      fp = open (path, 'w')
+      fp = open (path, 'wb')
       fp.write (reader.read ())
       fp.close ()
 
