@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -17,43 +17,22 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include <mobius/core/application.h>
 #include <mobius/disk/disk.h>
-#include <mobius/filesystem/filesystem.h>
-#include <mobius/imagefile/imagefile.h>
+#include <mobius/vfs/imagefile.h>
 #include <iostream>
 #include <unistd.h>
 
+void show_entry (const mobius::io::entry&);
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief show information about a filesystem entry
+//! \brief Shows streams
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-show_entry (const mobius::filesystem::entry entry)
+show_streams (const std::vector <mobius::io::stream>& streams)
 {
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // show entry metadata
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  std::cout << std::endl;
-  std::cout << "   [" << entry.get_inode () << "] ";
-
-  if (entry.is_deleted ())
-    std::cout << "<DEL> ";
-
-  std::cout << entry.get_path () << std::endl;
-
-  std::cout << "        size: " << entry.get_size () << std::endl;
-  std::cout << "        name: " << entry.get_name () << std::endl;
-  std::cout << "       atime: " << entry.get_last_access_time () << std::endl;
-  std::cout << "       mtime: " << entry.get_last_modification_time () << std::endl;
-  std::cout << "       ctime: " << entry.get_last_metadata_time () << std::endl;
-  std::cout << "      crtime: " << entry.get_creation_time () << std::endl;
-  std::cout << "       dtime: " << entry.get_deletion_time () << std::endl;
-
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // show entry streams
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   int i = 1;
   std::cout << "     streams:" << std::endl;
 
-  for (auto stream : entry.get_streams ())
+  for (const auto& stream : streams)
     {
       std::cout << "              stream " << i << ": " << stream.get_type ()
                 << ", size: " << stream.get_size () << ", name: " << stream.get_name ()
@@ -77,50 +56,143 @@ show_entry (const mobius::filesystem::entry entry)
 
       ++i;
     }
+}
 
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // show entry's children, if possible
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  if (entry.is_reallocated ())
-    return;
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Show file information
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+show_file (const mobius::io::file& f)
+{
+  // metadata
+  std::cout << std::endl;
+  std::cout << "   [" << f.get_inode () << "] ";
 
-  for (auto child : entry.get_children ())
+  if (f.is_deleted ())
+    std::cout << "<DEL> ";
+
+  std::cout << f.get_path () << std::endl;
+
+  std::cout << "        size: " << f.get_size () << std::endl;
+  std::cout << "        type: " << static_cast <int> (f.get_type ()) << std::endl;
+  std::cout << "        name: " << f.get_name () << std::endl;
+  std::cout << "        path: " << f.get_path () << std::endl;
+  std::cout << "       atime: " << f.get_access_time () << std::endl;
+  std::cout << "       mtime: " << f.get_modification_time () << std::endl;
+  std::cout << "       ctime: " << f.get_metadata_time () << std::endl;
+  std::cout << "      crtime: " << f.get_creation_time () << std::endl;
+  std::cout << "       dtime: " << f.get_deletion_time () << std::endl;
+  std::cout << "     bkptime: " << f.get_backup_time () << std::endl;
+
+  // parent
+  auto parent = f.get_parent ();
+
+  if (parent)
+    {
+      std::cout << " parent name: " << parent.get_name () << std::endl;
+      std::cout << " parent path: " << parent.get_path () << std::endl;
+    }
+
+  // streams
+  show_streams (f.get_streams ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Show folder information
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+show_folder (const mobius::io::folder& f)
+{
+  // metadata
+  std::cout << std::endl;
+  std::cout << " + [" << f.get_inode () << "] ";
+
+  if (f.is_deleted ())
+    std::cout << "<DEL> ";
+
+  std::cout << f.get_path () << std::endl;
+
+  std::cout << "        size: " << f.get_size () << std::endl;
+  std::cout << "        name: " << f.get_name () << std::endl;
+  std::cout << "        path: " << f.get_path () << std::endl;
+  std::cout << "       atime: " << f.get_access_time () << std::endl;
+  std::cout << "       mtime: " << f.get_modification_time () << std::endl;
+  std::cout << "       ctime: " << f.get_metadata_time () << std::endl;
+  std::cout << "      crtime: " << f.get_creation_time () << std::endl;
+  std::cout << "       dtime: " << f.get_deletion_time () << std::endl;
+
+  // parent
+  auto parent = f.get_parent ();
+
+  if (parent)
+    {
+      std::cout << " parent name: " << parent.get_name () << std::endl;
+      std::cout << " parent path: " << parent.get_path () << std::endl;
+    }
+
+  // streams
+  show_streams (f.get_streams ());
+
+  // children
+  for (const auto& child : f.get_children ())
     show_entry (child);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief show usage text
+//! \brief Show entry information
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+show_entry (const mobius::io::entry& entry)
+{
+  try
+    {
+      if (entry.is_file ())
+        show_file (entry.get_file ());
+
+      else
+        show_folder (entry.get_folder ());
+    }
+  catch (const std::exception& e)
+    {
+      std::cerr << "Warning: " << e.what () << std::endl;
+    }
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Show usage text
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 usage ()
 {
   std::cerr << std::endl;
-  std::cerr << "use: dirfs [OPTIONS] <URL>" << std::endl;
+  std::cerr << "Use: dirfs [OPTIONS] <URL>" << std::endl;
   std::cerr << "e.g: dirfs -t raw file://disk.raw" << std::endl;
   std::cerr << "     dirfs file://disk.ewf" << std::endl;
   std::cerr << std::endl;
-  std::cerr << "options are:" << std::endl;
+  std::cerr << "Options are:" << std::endl;
   std::cerr << "  -t imagefile type\t\t" << std::endl;
   std::cerr << std::endl;
-  std::cerr << "     type is:" << std::endl;
-  std::cerr << "       autodetect\ttry to autodetect imagefile type (default)" << std::endl;
+  std::cerr << "     Image file type can be:" << std::endl;
+  std::cerr << "       autodetect\tTry to autodetect imagefile type (default)" << std::endl;
 
-  for (const auto& i : mobius::imagefile::get_imagefile_types ())
+  for (const auto& i : mobius::vfs::get_imagefile_implementations ())
     std::cerr << "       " << i.type << "\t\t" << i.description << std::endl;
 
   std::cerr << std::endl;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief main function
+//! \brief Main function
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 int
 main (int argc, char **argv)
 {
   mobius::core::application app;
+  app.start ();
+
   std::cerr << app.get_name () << " v" << app.get_version () << std::endl;
   std::cerr << app.get_copyright () << std::endl;
-  std::cerr << "DirFS v1.0" << std::endl;
+  std::cerr << "DirFS v1.2" << std::endl;
   std::cerr << "by Eduardo Aguiar" << std::endl;
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -174,22 +246,25 @@ main (int argc, char **argv)
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   char drive = 'C';
 
-  auto filesystems = mobius::filesystem::get_filesystems (disk);
-  for (auto fs : filesystems)
+  for (const auto& fs : disk.get_filesystems ())
     {
       std::cout << std::endl;
 
       // shows filesystem metadata
-      for (const auto attr : fs.get_metadata ())
-        std::cout << "   " << attr.description << ": " << attr.value << std::endl;
+      for (const auto& p : fs.get_metadata ())
+        std::cout << "   " << p.first << ": " << p.second << std::endl;
 
       // shows entries
       std::cout << std::endl;
-      auto root = fs.get_root_entry ();
+
+      auto root = fs.get_root_folder ();
       root.set_path (std::string (1, drive) + ":");
-      show_entry (root);
+
+      show_folder (root);
+
       drive++;
     }
 
+  app.stop ();
   return EXIT_SUCCESS;
 }
